@@ -184,7 +184,7 @@ public class ResourceTracker {
      */
     public void addResource(URL location, Version version, DownloadOptions options, UpdatePolicy updatePolicy) {
         if (location == null)
-            throw new IllegalArgumentException("location==null");
+            throw new IllegalResourceDescriptorException("location==null");
         try {
             location = normalizeUrl(location, JNLPRuntime.isDebug());
         } catch (Exception ex) {
@@ -225,7 +225,7 @@ public class ResourceTracker {
      * not required as resources are reclaimed when the tracker is
      * collected.
      *
-     * @throws IllegalArgumentException if the resource is not being tracked
+     * @throws IllegalResourceDescriptorException if the resource is not being tracked
      */
     public void removeResource(URL location) {
         synchronized (resources) {
@@ -331,13 +331,13 @@ public class ResourceTracker {
         }
 
         DownloadEvent event = new DownloadEvent(this, resource);
-        for (int i = 0; i < l.length; i++) {
+        for (DownloadListener dl : l) {
             if (0 != ((ERROR | DOWNLOADED) & status))
-                l[i].downloadCompleted(event);
+                dl.downloadCompleted(event);
             else if (0 != (DOWNLOADING & status))
-                l[i].downloadStarted(event);
+                dl.downloadStarted(event);
             else if (0 != (CONNECTING & status))
-                l[i].updateStarted(event);
+                dl.updateStarted(event);
         }
     }
 
@@ -351,7 +351,7 @@ public class ResourceTracker {
      *
      * @param location the resource location
      * @return the resource, or null if it could not be downloaded
-     * @throws IllegalArgumentException if the resource is not being tracked
+     * @throws IllegalResourceDescriptorException if the resource is not being tracked
      * @see CacheUtil#isCacheable
      */
     public URL getCacheURL(URL location) {
@@ -378,7 +378,7 @@ public class ResourceTracker {
      *
      * @param location the resource location
      * @return a local file containing the resource, or null
-     * @throws IllegalArgumentException if the resource is not being tracked
+     * @throws IllegalResourceDescriptorException if the resource is not being tracked
      * @see CacheUtil#isCacheable
      */
     public File getCacheFile(URL location) {
@@ -418,7 +418,7 @@ public class ResourceTracker {
      * the cache.
      *
      * @throws IOException if there was an error opening the stream
-     * @throws IllegalArgumentException if the resource is not being tracked
+     * @throws IllegalResourceDescriptorException if the resource is not being tracked
      */
     public InputStream getInputStream(URL location) throws IOException {
         try {
@@ -442,15 +442,16 @@ public class ResourceTracker {
      * @param urls the resources to wait for
      * @param timeout the time in ms to wait before returning, 0 for no timeout
      * @return whether the resources downloaded before the timeout
-     * @throws IllegalArgumentException if the resource is not being tracked
+     * @throws IllegalResourceDescriptorException if the resource is not being tracked
      */
     public boolean waitForResources(URL urls[], long timeout) throws InterruptedException {
         Resource resources[] = new Resource[urls.length];
 
         synchronized (resources) {
             // keep the lock so getResource doesn't have to aquire it each time
-            for (int i = 0; i < urls.length; i++)
+            for (int i = 0; i < urls.length; i++) {
                 resources[i] = getResource(urls[i]);
+            }
         }
 
         if (resources.length > 0)
@@ -467,7 +468,7 @@ public class ResourceTracker {
      * @param timeout the timeout, or 0 to wait until completed
      * @return whether the resource downloaded before the timeout
      * @throws InterruptedException if another thread interrupted the wait
-     * @throws IllegalArgumentException if the resource is not being tracked
+     * @throws IllegalResourceDescriptorException if the resource is not being tracked
      */
     public boolean waitForResource(URL location, long timeout) throws InterruptedException {
         return wait(new Resource[] { getResource(location) }, timeout);
@@ -478,7 +479,7 @@ public class ResourceTracker {
      *
      * @param location the resource location
      * @return the number of bytes transferred
-     * @throws IllegalArgumentException if the resource is not being tracked
+     * @throws IllegalResourceDescriptorException if the resource is not being tracked
      */
     public long getAmountRead(URL location) {
         // not atomic b/c transferred is a long, but so what (each
@@ -490,7 +491,7 @@ public class ResourceTracker {
      * Returns whether a resource is available for use (ie, can be
      * accessed with the getCacheFile method).
      *
-     * @throws IllegalArgumentException if the resource is not being tracked
+     * @throws IllegalResourceDescriptorException if the resource is not being tracked
      */
     public boolean checkResource(URL location) {
         return getResource(location).isSet(DOWNLOADED | ERROR); // isSet atomic
@@ -504,7 +505,7 @@ public class ResourceTracker {
      * resource at a time to conserve system resources.
      *
      * @return true if the resource is already downloaded (or an error occurred)
-     * @throws IllegalArgumentException if the resource is not being tracked
+     * @throws IllegalResourceDescriptorException if the resource is not being tracked
      */
     public boolean startResource(URL location) {
         Resource resource = getResource(location);
@@ -517,7 +518,7 @@ public class ResourceTracker {
      * enqueues the resource if not already started.
      *
      * @return true if the resource is already downloaded (or an error occurred)
-     * @throws IllegalArgumentException if the resource is not being tracked
+     * @throws IllegalResourceDescriptorException if the resource is not being tracked
      */
     private boolean startResource(Resource resource) {
         boolean enqueue = false;
@@ -549,7 +550,7 @@ public class ResourceTracker {
      *
      * @param location the resource location
      * @return the number of bytes, or -1
-     * @throws IllegalArgumentException if the resource is not being tracked
+     * @throws IllegalResourceDescriptorException if the resource is not being tracked
      */
     public long getTotalSize(URL location) {
         return getResource(location).size; // atomic
@@ -605,7 +606,7 @@ public class ResourceTracker {
     private void queueResource(Resource resource) {
         synchronized (lock) {
             if (!resource.isSet(CONNECT | DOWNLOAD))
-                throw new IllegalArgumentException("Invalid resource state (resource: " + resource + ")");
+                throw new IllegalResourceDescriptorException("Invalid resource state (resource: " + resource + ")");
 
             queue.add(resource);
             startThread();
@@ -868,8 +869,7 @@ public class ResourceTracker {
                     resource.toString() + " : " + urls);
         }
         URL bestUrl = null;
-        for (int i = 0; i < urls.size(); i++) {
-            URL url = urls.get(i);
+        for (URL url : urls) {
             try {
                 URLConnection connection = url.openConnection();
                 connection.addRequestProperty("Accept-Encoding", "pack200-gzip, gzip");
@@ -999,8 +999,7 @@ public class ResourceTracker {
         Resource result = null;
         int score = Integer.MAX_VALUE;
 
-        for (int i = 0; i < source.size(); i++) {
-            Resource resource = source.get(i);
+        for (Resource resource : source) {
             boolean selectable = false;
 
             synchronized (resource) {
@@ -1011,9 +1010,10 @@ public class ResourceTracker {
             if (selectable) {
                 int activeCount = 0;
 
-                for (int j = 0; j < active.size(); j++)
-                    if (active.get(j) == resource.getTracker())
+                for (ResourceTracker rt : active) {
+                    if (rt == resource.getTracker())
                         activeCount++;
+                }
 
                 // try to spread out the downloads so that a slow host
                 // won't monopolize the downloads
@@ -1030,19 +1030,17 @@ public class ResourceTracker {
     /**
      * Return the resource matching the specified URL.
      *
-     * @throws IllegalArgumentException if the resource is not being tracked
+     * @throws IllegalResourceDescriptorException if the resource is not being tracked
      */
     private Resource getResource(URL location) {
         synchronized (resources) {
-            for (int i = 0; i < resources.size(); i++) {
-                Resource resource = resources.get(i);
-
+            for (Resource resource : resources) {
                 if (CacheUtil.urlEquals(resource.location, location))
                     return resource;
             }
         }
 
-        throw new IllegalArgumentException("Location does not specify a resource being tracked.");
+        throw new IllegalResourceDescriptorException("Location does not specify a resource being tracked.");
     }
 
     /**
@@ -1058,8 +1056,9 @@ public class ResourceTracker {
         long startTime = System.currentTimeMillis();
 
         // start them downloading / connecting in background
-        for (int i = 0; i < resources.length; i++)
-            startResource(resources[i]);
+        for (Resource resource : resources) {
+            startResource(resource);
+        }
 
         // wait for completion
         while (true) {
@@ -1067,11 +1066,11 @@ public class ResourceTracker {
 
             synchronized (lock) {
                 // check for completion
-                for (int i = 0; i < resources.length; i++) {
+                for (Resource resource : resources) {
                     //NetX Deadlocking may be solved by removing this
                     //synch block.
-                    synchronized (resources[i]) {
-                        if (!resources[i].isSet(DOWNLOADED | ERROR)) {
+                    synchronized (resource) {
+                        if (!resource.isSet(DOWNLOADED | ERROR)) {
                             finished = false;
                             break;
                         }
